"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.withDenormalization = exports.CouldNotDereferenceError = exports.CACHE_NORMALIZATION_FOREIGN_KEY_REGEXP = void 0;
exports.CACHE_NORMALIZATION_FOREIGN_KEY_REGEXP = /^\.cache\.ref\..+$/;
class CouldNotDereferenceError extends Error {
    constructor({ referenceKey }) {
        super(`Could not find the normalized value referenced by key ${referenceKey} in the cache during denormalization. Has it expired?`);
    }
}
exports.CouldNotDereferenceError = CouldNotDereferenceError;
const recursivelyHydrateCachedValueReferences = ({ get, value, }) => __awaiter(void 0, void 0, void 0, function* () {
    // if the value is a string, see if its a reference and hydrate if so
    if (typeof value === 'string') {
        const isAReferenceKey = exports.CACHE_NORMALIZATION_FOREIGN_KEY_REGEXP.test(value);
        // if its not a reference key, then its as hydrated as it can get
        if (!isAReferenceKey)
            return value;
        // if it is a reference key, get the referenced value!
        const referencedValue = yield get(value);
        if (!referencedValue)
            throw new CouldNotDereferenceError({ referenceKey: value }); // !: this error should be caught, warned on, and whole cache.output should be simply treated as invalid -> non existent, due to broken reference
        return referencedValue;
    }
    // since its not a string, if its not an object either, no chance of any further hydration
    if (typeof value !== 'object' || value === null)
        return { value, references: {} };
    // since its an object, attempt hydration on each of the keys
    const entries = Object.entries(value);
    const entriesHydrated = yield Promise.all(entries.map(([key, val]) => __awaiter(void 0, void 0, void 0, function* () {
        const valHydrated = yield recursivelyHydrateCachedValueReferences({
            get,
            value: val,
        });
        return [key, valHydrated]; // use the hydrated value
    })));
    const valueHydrated = Array.isArray(value)
        ? entriesHydrated.map((entry) => entry[1]) // if original value was an array, then the entry keys are just the index positions
        : Object.fromEntries(entriesHydrated);
    return valueHydrated;
});
/**
 * a utility which dereferences values in a cached output, ensuring the output is fully hydrated
 *
 * specifically
 * - looks up the value of each reference in the .get output
 * - replaces the reference in the .get output with the true value
 * - does so recursively
 *
 * usecases
 * - normalizing data that changes from multiple operates into its own cache key, to enable automatic cache updates
 *
 * note
 * - this is typically used to denormalize output normalized with the `withNormalization` method
 */
const withDenormalization = (cache) => {
    return Object.assign(Object.assign({}, cache), { set: (...args) => __awaiter(void 0, void 0, void 0, function* () { return cache.set(...args); }), get: (key) => __awaiter(void 0, void 0, void 0, function* () {
            // get the cached value
            const found = yield cache.get(key);
            // if nothing found, nothing more to do
            if (!found)
                return found;
            // recursively walk through all object keys and hydrate any references found
            const hydrated = (yield recursivelyHydrateCachedValueReferences({
                get: (thisKey) => __awaiter(void 0, void 0, void 0, function* () { return cache.get(thisKey); }),
                value: found,
            }));
            // return the hydrated value
            return hydrated;
        }) });
};
exports.withDenormalization = withDenormalization;
//# sourceMappingURL=withDenormalization.js.map