"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const domain_objects_1 = require("domain-objects");
const normalizeDomainObjectReferences_1 = require("../normalizers/normalizeDomainObjectReferences");
const withNormalization_1 = require("./withNormalization");
const withSerialization_1 = require("./withSerialization");
class Ship extends domain_objects_1.DomainEntity {
}
Ship.unique = ['isid'];
class Geocode extends domain_objects_1.DomainValueObject {
}
describe('withNormalization', () => {
    beforeEach(() => jest.clearAllMocks());
    const exampleStore = {};
    const cacheGetMock = jest.fn((key) => exampleStore[key]);
    const cacheSetMock = jest.fn((key, value) => (exampleStore[key] = value));
    const cache = {
        get: cacheGetMock,
        set: cacheSetMock,
    };
    const cacheWithNormalization = (0, withNormalization_1.withNormalization)(cache, {
        normalize: normalizeDomainObjectReferences_1.normalizeDomainObjectReferences,
    });
    it('should be able to normalize direct reference set to the cache', () => __awaiter(void 0, void 0, void 0, function* () {
        const ship = new Ship({ isid: '__isid__', name: 'boaty mcboatface' });
        const shipReferenceKey = (0, normalizeDomainObjectReferences_1.getCacheReferenceKeyForDomainObject)(ship);
        // should set the reference and normalized value
        yield cacheWithNormalization.set('__key__', ship);
        expect(cacheSetMock).toHaveBeenCalledWith(shipReferenceKey, ship, expect.any(Object));
        expect(cacheSetMock).toHaveBeenCalledWith('__key__', shipReferenceKey, undefined);
        // should get the denormalized value
        const found = yield cacheWithNormalization.get('__key__');
        expect(found).toEqual(ship);
    }));
    it('should be able to normalize nested references set to the cache', () => __awaiter(void 0, void 0, void 0, function* () {
        const ship = new Ship({ isid: '__isid__', name: 'boaty mcboatface' });
        const shipReferenceKey = (0, normalizeDomainObjectReferences_1.getCacheReferenceKeyForDomainObject)(ship);
        // should set the reference and normalized value
        yield cacheWithNormalization.set('__key__', { ship, color: 'green' });
        expect(cacheSetMock).toHaveBeenCalledWith(shipReferenceKey, ship, expect.any(Object));
        expect(cacheSetMock).toHaveBeenCalledWith('__key__', { ship: shipReferenceKey, color: 'green' }, undefined);
        // should get the denormalized value
        const found = yield cacheWithNormalization.get('__key__');
        expect(found).toEqual({ ship, color: 'green' });
    }));
    it('should be able to normalize deeply nested references set to the cache', () => __awaiter(void 0, void 0, void 0, function* () {
        const ship = new Ship({ isid: '__isid__', name: 'boaty mcboatface' });
        const shipReferenceKey = (0, normalizeDomainObjectReferences_1.getCacheReferenceKeyForDomainObject)(ship);
        const geocode = new Geocode({ latitude: 821, longitude: 1215 });
        const geocodeReferenceKey = (0, normalizeDomainObjectReferences_1.getCacheReferenceKeyForDomainObject)(geocode);
        // should set the reference and normalized value
        yield cacheWithNormalization.set('__key__', {
            ship,
            color: 'green',
            position: { geocode },
        });
        expect(cacheSetMock).toHaveBeenCalledWith(shipReferenceKey, ship, expect.any(Object));
        expect(cacheSetMock).toHaveBeenCalledWith(geocodeReferenceKey, geocode, expect.any(Object));
        expect(cacheSetMock).toHaveBeenCalledWith('__key__', {
            ship: shipReferenceKey,
            color: 'green',
            position: { geocode: geocodeReferenceKey },
        }, undefined);
        // should get the denormalized value
        const found = yield cacheWithNormalization.get('__key__');
        expect(found).toEqual({
            ship,
            color: 'green',
            position: { geocode },
        });
    }));
    it('should be able to normalize deeply nested references set to a cache with serialization', () => __awaiter(void 0, void 0, void 0, function* () {
        const cacheOfString = {
            get: cacheGetMock,
            set: cacheSetMock,
        };
        const cacheOfStringWithSerialization = (0, withSerialization_1.withSerialization)(cacheOfString, {
            serialize: domain_objects_1.serialize,
            deserialize: (value) => (0, domain_objects_1.deserialize)(value, { with: [Ship, Geocode] }),
        });
        const cacheOfStringWithSerializationAndNormalization = (0, withNormalization_1.withNormalization)(cacheOfStringWithSerialization, {
            normalize: normalizeDomainObjectReferences_1.normalizeDomainObjectReferences,
        });
        const ship = new Ship({ isid: '__isid__', name: 'boaty mcboatface' });
        const shipReferenceKey = (0, normalizeDomainObjectReferences_1.getCacheReferenceKeyForDomainObject)(ship);
        const geocode = new Geocode({ latitude: 821, longitude: 1215 });
        const geocodeReferenceKey = (0, normalizeDomainObjectReferences_1.getCacheReferenceKeyForDomainObject)(geocode);
        // should set the reference and normalized value
        yield cacheOfStringWithSerializationAndNormalization.set('__key__', {
            ship,
            color: 'green',
            position: { geocode },
        });
        expect(cacheSetMock).toHaveBeenCalledWith(shipReferenceKey, (0, domain_objects_1.serialize)(ship), expect.any(Object));
        expect(cacheSetMock).toHaveBeenCalledWith(geocodeReferenceKey, (0, domain_objects_1.serialize)(geocode), expect.any(Object));
        expect(cacheSetMock).toHaveBeenCalledWith('__key__', (0, domain_objects_1.serialize)({
            ship: shipReferenceKey,
            color: 'green',
            position: { geocode: geocodeReferenceKey },
        }), undefined);
        // should get the denormalized value
        const found = yield cacheOfStringWithSerializationAndNormalization.get('__key__');
        expect(found).toEqual({
            ship,
            color: 'green',
            position: { geocode },
        });
    }));
});
//# sourceMappingURL=withNormalization.test.js.map